	PROGRAM CMPTOLMO
C
C	PERFORMS LUMPING.  CONVERTS .CMP FILES TO .LMO FILES
C
C	COMMAND LINE:
C
C		P1 = .CMP INPUT FILE NAME.
C		P2 = .LMO OUTPUT FILE NAME (FOR CONCENTRATIONS)
C		P3 = .RXP OUTPUT FILE NAME (FOR PARAMETERS)
C		     (IF NEITHER ARE GIVEN,  BOTH A .LMO AND .RXP FILE
C		      PRODUCED, WITH SAME NAME AS .CMP FILE.
C		P4 (OPTIONAL) = PARAMETER FILE (READ AFTER MODELING.PRM).
C			OR, IF .PRM FILE NOT FOUND, LOOKS FOR .RXP FILE
C			WITH SAME NAME. (SAME AS LUMPPRE= IN MODELING.PRM)
C
C	PARAMETERS USED (FROM MODELING.PRM OR PARAMETER FILE):
C
C		LPC=		LUMPING CONTROL FILE
C		MECH=		UIC FOR FILES REFERENCED IN .LPC FILE.
C		DMS= (OR VOC=)	DMS ASSIGNMENT FILE (DEFAULT=DMS.PRM)
C		CMP=		UIC FOR .CMP FILES INPUT
C		RXP=		UIC FOR .RXP FILES OUTPUT
C		TEMP=		LUMPING TEMPERATURE (DEFAULT=300)
C		LUMPPRE=	PREFIX FOR OUTPUT CONC'S
C		LUMP=NORENAME	DO NOT RE-NAME SPECIES (IGNORED IN THIS VERSION)
C		LUMP=RENAME	OK TO RE-NAME SPECIES (DEFAULT)
C		LUMP=NOPARM	DO NOT OUTPUT PARAMETERS IN .RXP FILE.
C		LUMP=OUTPARM	OUTPUT PARAMETERS (DEFAULT)
C
C
C	LUMPING CONTROL SPECIFICATIONS (READ IN BY RDLUMP),
C	AND SPECIFICATIONS FOR LUMPED GROUPS.
C
	INCLUDE 'LUMPSPEC.FOR'
C
C
C	SPECIFICATIONS OF ARRAYS USED FOR DETAILED MODEL SPECIES
C	CALCULATED FOR INPUT EMISSIONS PROFILE.
C
C	MAXNMS	MAX. NO DETAILED MODEL SPECIES
C	NMS	NO. DETAILED MODEL SPECIES IN PROFILE
C	MSNAM	MODEL SPECIES NAME
C	MSLVL	MOLES MODEL SPECIES
C	MSNC	NUMBER OF CARBONS
C	MSMW	MOLECULAR WEIGHT
C
	PARAMETER (MAXNMS=2000)
	INTEGER NMS
	CHARACTER*8 MSNAM(MAXNMS)
	REAL*4 MSLVL(MAXNMS),MSNC(MAXNMS),MSMW(MAXNMS)
C
C
C	LOCAL SPECIFICATIONS
C
	CHARACTER LPCFIL*40,CMPUIC*40,RXPUIC*40,CMPFIL*80,RXPFIL*80,
     &	 DMSFIL*40,PRMFIL*40,INLIN*80,NAMEIN*8,LMOFIL*80
C
	LOGICAL ERRIND,OUTPRM
C	LOGICAL RENAME
C
C
C	GET AND DECODE COMMAND LINE
C
	CALL GETARG(1,CMPFIL)
	LCMPFL=LNBLNK(CMPFIL)
	CALL GETARG(2,LMOFIL)
	LLMOFL=LNBLNK(LMOFIL)
	CALL GETARG(3,RXPFIL)
	LRXPFL=LNBLNK(RXPFIL)
	CALL GETARG(4,PRMFIL)
	LPRMFL=LNBLNK(PRMFIL)
	CALL GETARG(5,LPRECH)
	IF (LPRECH(1:1).EQ.' ') THEN
		LPRE=0
	ELSE
		LPRE=LNBLNK(LPRECH)
	ENDIF
C
	IF (CMPFIL(1:1).EQ.' ') THEN
		WRITE (6,*) 'Useage:   CMPTOLMO cmpfile [lmofile] ',
     &		 '[rxpfile] [prmfile|lpcfile]'
		WRITE (6,*) 'If neither .LMO nor .RXP file names given,'
     &		 ,' these are given name of .CMP file.'
		GOTO 899
	ENDIF
	OUTPRM=.TRUE.
	IF (LMOFIL(1:1).EQ.' ') THEN
		LMOFIL=CMPFIL
		LLMOFL=LCMPFL
		RXPFIL=CMPFIL
		LRXPFL=LCMPFL
	ELSEIF (RXPFIL(1:1).EQ.' ') THEN
		OUTPRM=.FALSE.
	ELSEIF (RXPFIL(1:4).EQ.'NUL ') THEN
		OUTPRM=.FALSE.
		LRXPFL=0
		RXPFIL=' '
	ENDIF
C
C	INITIALIZE LUMPING PARAMETERS
C	INITIALIZE LUMPING AND TEMPERATURE AS BEING UNDEFINED.
C
	NOLPC=.TRUE.
	TEMPR=0.0
C	RENAME=.TRUE.
C
C	DEFAULT IS TO COMPUTE T-DEPENDENT KP'S
C
	THIGH=330.0
	TLOW=270.0
C
	CMPUIC=' '
	LCMPUC=0
	LPCFIL=' '
	LLPCFL=0
	LPCFIL=' '
	LLPCFL=0
	MECUIC=' '
	DMSFIL='DMS.PRM'
	LDMSFL=7
	RXPUIC=' '
	LRXPUC=0
C
C	GET PARAMETERS FROM MODELING.PRM
C
	OPEN (UNIT=1,FILE='MODELING.PRM',STATUS='UNKNOWN')
	IND=0
C
   10	READ (1,1,END=12) INLIN
    1	FORMAT (A)
	L=LNBLNK(INLIN)
C
	IF (INLIN(1:4).EQ.'LPC=') THEN
		LLPCFL=L-4
		IF (L.GT.4) THEN
			LPCFIL=INLIN(5:L)
		ELSE
			LPCFIL=' '
		ENDIF
	ELSEIF (INLIN(1:5).EQ.'MECH=') THEN
		IF (L.GT.5) THEN
			MECUIC=INLIN(6:L)
		ELSE
			MECUIC=' '
		ENDIF
	ELSEIF (INLIN(1:4).EQ.'CMP=') THEN
		LCMPUC=L-4
		IF (L.GT.4) THEN
			CMPUIC=INLIN(5:L)
		ELSE
			CMPUIC=' '
		ENDIF
	ELSEIF (INLIN(1:4).EQ.'RXP=') THEN
		LRXPUC=L-4
		IF (L.GT.4) THEN
			RXPUIC=INLIN(5:L)
		ELSE
			RXPUIC=' '
		ENDIF
	ELSEIF (INLIN(1:4).EQ.'DMS='.OR.INLIN(1:4).EQ.'VOC=') THEN
		LDMSFL=L-4
		IF (L.GT.4) THEN
			DMSFIL=INLIN(5:L)
		ELSE
			DMSFIL=' '
		ENDIF
	ELSEIF (INLIN(1:5).EQ.'TEMP=') THEN
		CALL PARSE0 (1,INLIN(6:),75)
		CALL PARSER (1,TEMPR,ERRIND,INLIN)
		IF (ERRIND.OR.TEMPR.LE.0.0) THEN
			WRITE (6,*) 'WARNING:  BAD "TEMP=" INPUT IN',
     &			 ' PARM FILE IGONRED.'
			TEMPR=0.0
			GOTO 10
		ENDIF
	ELSEIF (INLIN(1:8).EQ.'LUMPPRE=') THEN
		IF (L.EQ.8) THEN
			LPRE=0
			LPRECH=' '
		ELSE
			LPRE=L-8
			LPRECH=INLIN(9:12)
		ENDIF
C	ELSEIF (INLIN(1:13).EQ.'LUMP=NORENAME') THEN
C		RENAME=.FALSE.
C	ELSEIF (INLIN(1:11).EQ.'LUMP=RENAME') THEN
C		RENAME=.TRUE.
	ELSEIF (INLIN(1:11).EQ.'LUMP=NOPARM') THEN
		OUTPRM=.FALSE.
	ELSEIF (INLIN(1:12).EQ.'LUMP=OUTPARM') THEN
		OUTPRM=.TRUE.
	ENDIF
	GOTO 10
C
   12	CLOSE (UNIT=1)
C
C	IF PARAMETER FILE NAMED, PASS THROUGH AGAIN AND READ
C	INPUT THERE AS IF FROM MODELING.PRM
C
	IF (IND.EQ.0.AND.PRMFIL(1:1).NE.' ') THEN
		INLIN=PRMFIL(1:LPRMFL)
		INLIN(LPRMFL+1:LPRMFL+4)='.PRM'
		OPEN (UNIT=1,FILE=INLIN,STATUS='OLD',ERR=14)
		IND=1
		GOTO 10
C
C	IF P3 CAN'T BE OPENED AS A PARM FILE, IT MUST BE A .LPC FILE
C	
   14		LPCFIL=PRMFIL
		LLPCFL=LPRMFL
	ENDIF
C
C
C	OPEN INPUT FILE.
C
	IF (LCMPUC.EQ.0) THEN
		INLIN=CMPFIL
		L=LCMPFL
	ELSE
		L=LCMPUC+LCMPFL
		INLIN=CMPUIC
		INLIN(LCMPUC+1:L)=CMPFIL
	ENDIF
	INLIN(L+1:L+4)='.CMP'
	CMPFIL=INLIN
	LCMPFL=L+4
	OPEN (UNIT=1,FILE=INLIN,STATUS='OLD',ERR=810)
C
C
C	INITIALIZE TEMPERATURE, PREFIX
C
	IF (TEMPR.EQ.0.0) TEMPR=300.0
	INLIN=LPCFIL(1:LLPCFL)
	INLIN(LLPCFL+1:LLPCFL+4)='.LPC'
	L=LLPCFL
C
C	OPEN .LPC FILE.  NOTE "2" AS 3RD ARGUMENT MEANS T, LUMPPRE
C	GIVEN IN LPC FILE IS IGNORED.
C
	OPEN (UNIT=2,FILE=INLIN,STATUS='OLD',ERR=810)
	CALL RDLUMP (-2,3,2)
C
C
C	READ DETALED MODEL SPECIES SPECIFICATIONS FROM DMS FILE
C
	OPEN (UNIT=2,FILE=DMSFIL(1:LDMSFL),STATUS='OLD',ERR=820)
C
	NMS=0
C
   30	READ (2,1,END=38) INLIN
	IF (INLIN(1:1).NE.'.') GOTO 30
   32	READ (2,1,END=38) INLIN
	IF (INLIN(1:1).EQ.'!') GOTO 32
	IF (INLIN(1:1).EQ.' ') GOTO 32
	IF (INLIN(1:1).EQ.'	') GOTO 32
	NMS=NMS+1
	IF (NMS.GT.MAXNMS) THEN
		WRITE (6,*) 'Too many DMS in ',DMSFIL(1:LDMSFL)
		GOTO 899
	ENDIF
	READ (INLIN,'(A8,2X,F4.0,F9.2)',ERR=825) MSNAM(NMS),MSNC(NMS),
     &	 MSMW(NMS)
	MSLVL(NMS)=0.0
	GOTO 32
C
   38	CLOSE (UNIT=2)
	IF (NMS.EQ.0) THEN
		WRITE (6,*) 'No DMS in ',DMSFIL(1:LDMSFL)
		GOTO 899
	ENDIF
C
C
C	OPEN OUTPUT FILES.  COMMENTS GIVE .LPC FILE AND
C	LUMPING TEMPERATURE.
C
        IF (LRXPUC.NE.0) THEN
                LLMOFL=LRXPUC+LLMOFL+4
                INLIN=RXPUIC
                INLIN(LRXPUC+1:)=LMOFIL
                LMOFIL=INLIN
	ELSE
		LLMOFL=LLMOFL+4
        ENDIF
        LMOFIL(LLMOFL-3:LLMOFL)='.LMO'
        OPEN (UNIT=2,FILE=LMOFIL(1:LLMOFL),STATUS='UNKNOWN')
C
        IF (OUTPRM) THEN
                IF (LRXPUC.NE.0) THEN
                        LRXPFL=LRXPUC+LRXPFL+4
                        INLIN=RXPUIC
                        INLIN(LRXPUC+1:)=RXPFIL
                        RXPFIL=INLIN
		ELSE
			LRXPFL=LRXPFL+4
                ENDIF
                RXPFIL(LRXPFL-3:LRXPFL)='.RXP'
                OPEN (UNIT=3,FILE=RXPFIL(1:LRXPFL),STATUS='UNKNOWN')
        ENDIF
C
	CALL TMARK (INLIN)
	WRITE (2,102) LMOFIL(LRXPUC+1:LLMOFL),INLIN(1:16),
     &	 CMPFIL(1:LCMPFL),LPCFIL(1:LLPCFL),DMSFIL(1:LDMSFL),TEMPR
	IF (OUTPRM) WRITE (3,102) RXPFIL(LRXPUC+1:LRXPFL),INLIN(1:16),
     &	 CMPFIL(1:LCMPFL),LPCFIL(1:LLPCFL),DMSFIL(1:LDMSFL),TEMPR
  102	FORMAT ('! File = ',A,'.   Time created = ',A/'!'
     &         /'! Input file            = ',A
     &         /'! Lumping control file  = ',A,'.LPC'
     &         /'! DMS assignment file   = ',A
     &         /'! Lumping temperature   = ',F8.2/'!')
C
C
C	READ DMS'S FROM INPUT FILE.
C
	FAC=1.0
	TOTNC=0.0
	TOTWT=0.0
	TOTUNK=0.0
C
  200	READ (1,1,END=290) INLIN
	IF (INLIN(1:1).EQ.'!') GOTO 200
	IF (INLIN(1:1).EQ.' ') GOTO 200
C
	CALL PARSE0 (2,INLIN,80)
	CALL PARSE1 (1,NAMEIN,L,8,INLIN)
	CALL PARSER (2,X,ERRIND,INLIN)
	IF (ERRIND) GOTO 280
C
	IF (NAMEIN(1:4).EQ.'#FAC') THEN
		IF (X.LE.0.0) GOTO 280
		FAC=X
	ELSE
		X=X*FAC
	ENDIF
C
	DO 210 I=1,NMS
	IF (NAMEIN.EQ.MSNAM(I)) THEN
		MSLVL(I)=MSLVL(I)+X
		TOTNC=TOTNC+(X*MSNC(I))
		TOTWT=TOTWT+(X*MSMW(I))
		GOTO 200
	ENDIF
  210	CONTINUE
C
	WRITE (6,*) 'Unrecognized DMS = ',NAMEIN
	WRITE (2,220) NAMEIN,X
  220	FORMAT ('! DMS not found: ',A8,', level = ',1PE12.3)
	TOTUNK=TOTUNK+X
	GOTO 200
C
  280	WRITE (6,*) 'Warinig:  Bad .CMP input line = ''',
     &	 INLIN(1:LNBLNK(INLIN)),'''.  Ignored.'
	GOTO 200
C
  290	CLOSE (UNIT=1)
C
	IF (TOTUNK.NE.0.0) WRITE 
     &	 (2,'(''!''/''UNKNOWNS'',1PE12.3/''!'')') TOTUNK
	IF (TOTNC.EQ.0) THEN
		WRITE (6,*) 'No recognized non-zero DMS'
		WRITE (2,*) '! No recognized non-zero DMS'
	ELSE
		WRITE (2,292) TOTNC,TOTWT,TOTWT/TOTNC
  292		FORMAT ( '! Total no. carbons     = ',F8.4
     &                  /'! Total mass            = ',F7.3
     &                  /'! Average MW            = ',F6.2/'!')
		IF (OUTPRM) WRITE (3,293) TOTWT/TOTNC
  293		FORMAT ( '! Average MW            = ',F6.2/'!')
C
C		FIGURE LUMPING FROM MODEL SPECIES
C
		INTFMT=.TRUE.
		IF (OUTPRM) THEN
			I=3
		ELSE
			I=0
		ENDIF
		CALL LMPSUB (2,I,NMS,MSNAM,MSLVL,MSNC,MSMW)
	ENDIF
C
C	DONE
C
	WRITE (6,*) 'File = ',LMOFIL(1:LLMOFL),' created.'
	IF (OUTPRM) WRITE (6,*) 'File = ',RXPFIL(1:LRXPFL),
     &	 ' created.'
	CALL EXIT (0)
C
C
C	ERROR EXIT.  DELETE STATUS.TMP FILE.
C
  810	WRITE (6,*) 'File = ',INLIN(1:L+4),' does not exist.'
	GOTO 899
  820	WRITE (6,*) 'Error opening ',DMSFIL(1:LDMSFL),' for DMS',
     &	 ' assignments.'
	GOTO 899
  825	WRITE (6,*) 'Error reading data in ',DMSFIL(1:LDMSFL),'.'
	WRITE (6,*) 'Bad line = ',INLIN(1:LNBLNK(INLIN))
C
  899	CALL EXIT (2)
	END
