C	LMPSUB.FOR
C
C	SUBROUTINE TO CALCULATE TOTAL MOLES AND AVERAGE KINETIC
C	AND MECHANISTIC PARAMETERS FOR LUMPED MODEL SPECIES GIVEN
C	DETAILED MODEL SPECIES COMPOSITIONS (ARGUMENTS) AND
C	DATA READ FROM LUMPING CONTROL FILES (IN COMMON)
C
C	7/90 VERSION.  USES ONLY DMS RATE CONSTANTS GIVEN IN MECHANISM
C	ASSIGNMENT FILES.
C
C	OUTRXN OPTION: OUTPUTS ONLY PRODUCT NAMES
C
C	ARGUMENTS:
C		NMS	NO. DETAILED MODEL SPECIES IN COMOSITION
C		MSNAM	MODEL SPECIES NAME
C		MSLVL	MOLES MODEL SPECIES
C		MSNC	NUMBER OF CARBONS
C		MSMW	MOLECULAR WEIGHT
C
C		UNITA	OUTPUT UNIT FOR LUMPED SPECIES LEVELS
C		UNITB	OUTPUT UNIT FOR LUMPED SPECIES PARAMETERS
C
C		
	SUBROUTINE LMPSUB (UNITA,UNITB,NMS,MSNAM,MSLVL,MSNC,MSMW)
C
	CHARACTER*8 MSNAM(*)
	REAL*4 MSLVL(*),MSNC(*),MSMW(*)
	INTEGER UNITA,UNITB,NMS
C
C	COMMON SPECIFICATIONS
C
	INCLUDE 'LUMPSPEC.FOR'
C
C	GIVES MODEL SPECIES ID NO FOR A NON-LUMPED SPECIES.
C
	INTEGER NLSMOD(MAXNLS)
C
C	LOCAL SPECIFICATIONS
C
	CHARACTER*8 MSNAM2(MAXDMS)
	REAL*4 MSLVL2(MAXDMS),MSNC2(MAXDMS),MSMW2(MAXDMS)
C
	LOGICAL VARYT
	REAL*4 NROGMS
	CHARACTER NAMEIN*8
C
C
	VARYT=THIGH*TLOW.NE.0.0
C
C	INITIALIZE LUMPED GROUPS FOR THIS FILE.  NOTE THAT KOH
C	AND PARM ARRAYS CONTAIN TOTALS, WHICH WILL BE NORMALIZED
C	BY TOTAL AMOUNT IN GROUP TO GET AVERAGE AFTER INPUT DONE.
C	FIRST ONE OF BOTH ARE 'INERT' GROUPS.
C
	NNLS=1
	NLSNAM(1)='INERT'
	NLSLVL(1)=0.0
	NLSNC(1)=0.0
	NLSMAS(1)=0.0
	NLSMOD(1)=0
C
	DO 210 I=1,NGRP
	TOTGRP(I)=0.0
	RKGRP(I)=0.0
	RCTGRP(I)=0.0
	NCGRP(I)=0.0
	NCRGRP(I)=0.0
	MASGRP(I)=0.0
	MSRGRP(I)=0.0
	DO 210 J=1,NAAPRM
  210	PRMGRP(J,I)=0.0
C
C	PRE-PROCESS MODEL SPECIES IF NEEDED
C	IF MIXTURE DMS HAS ONE CARBON, THEN IT IS ASSUMED TO BE A COMPLEX
C	MIXTURE AND COMPONENT NC AND MWT ARE NOT CHANGED.  OTHERWISE,
C	ALL COMPONENTS IN MIXTURE TREATED AS HAVING SAME NC AND MWT,
C	ADJUSTED BY TOTAL MOLES DIFFERENCE
C
	IF (NMIXS.NE.0) THEN
		NMS2=0
		DO 2102 IMS=1,NMS
		F=MSLVL(IMS)
		IF (F.EQ.0.0) GOTO 2102
		NAMEIN=MSNAM(IMS)
		FNCI=MSNC(IMS)
		FMWI=MSMW(IMS)
		DO 2104 IMIX=1,NMIXS
		IF (MIXNAM(IMIX).EQ.NAMEIN) THEN
			IF (NMS2+NMIXI(IMIX).GT.MAXDMS) THEN
				WRITE (6,*) 'TOO MANY SPECIES IN MIX'
				WRITE (6,*) 'INCREASE MAXDMS IN LUMPSPEC'
				CALL EXIT (2)
			ENDIF
			IF (FNCI.EQ.1.0) THEN
				DO 2107 I=MIXI1(IMIX),MIXI1(IMIX)+NMIXI(IMIX)-1
				NMS2=NMS2+1
				MSNAM2(NMS2)=MIXCMP(I)
				MSLVL2(NMS2)=F*MIXFAC(I)
				DO 2108 J=1,NMS
				IF (MIXCMP(I).EQ.MSNAM(J)) THEN
					MSNC2(NMS2)=MSNC(J)
					MSMW2(NMS2)=MSMW(J)
					GOTO 2107
				ENDIF
 2108				CONTINUE
				MSNC2(NMS2)=FNCI
				MSMW2(NMS2)=FMWI
 2107				CONTINUE
			ELSE
				F2=MIXMOL(IMIX)
				DO 2106 I=MIXI1(IMIX),MIXI1(IMIX)+NMIXI(IMIX)-1
				NMS2=NMS2+1
				MSNAM2(NMS2)=MIXCMP(I)
				MSLVL2(NMS2)=F*MIXFAC(I)
				MSNC2(NMS2)=FNCI/F2
 2106				MSMW2(NMS2)=FMWI/F2
			ENDIF
			GOTO 2102
		ENDIF
 2104		CONTINUE
		IF (NMS2.EQ.MAXDMS) THEN
			WRITE (6,*) 'TOO MANY SPECIES IN MIX'
			WRITE (6,*) 'INCREASE MAXDMS IN LUMPSPEC'
			CALL EXIT (2)
		ENDIF
		NMS2=NMS2+1
		MSNAM2(NMS2)=NAMEIN
		MSLVL2(NMS2)=F
		MSNC2(NMS2)=FNCI
		MSMW2(NMS2)=FMWI
 2102		CONTINUE
		NMS=NMS2
	ENDIF
C
C	PROCESS EACH MODEL SPECIES ENTRY, AND ADD IT TO GROUP
C	IGNORE ZERO LEVEL ENTRIES
C
	DO 220 IMS=1,NMS
	IF (NMIXS.EQ.0) THEN
		NAMEIN=MSNAM(IMS)
		F=MSLVL(IMS)
		IF (F.EQ.0.0) GOTO 220
		FNCI=MSNC(IMS)
		FMWI=MSMW(IMS)
	ELSE
		NAMEIN=MSNAM2(IMS)
		F=MSLVL2(IMS)
		FNCI=MSNC2(IMS)
		FMWI=MSMW2(IMS)
	ENDIF
C
C	FIRST REPLACE BY SUBSTITUTIONS, IF ANY.  (NOTE:  REPLACED SUCH
C	THAT TOTAL NUMBER OF CARBONS AND WEIGHT IS THE SAME.)
C	RE-ITERATES UNTIL NO CHANGE AFTER FULL PASS.
C
  221	DO 222 I=1,NSUB
	IF (NAMEIN.EQ.SUBNAM(I)) THEN
		NAMEIN=SUBOUT(I)
		F=F*SUBFAC(I)
		FNCI=FNCI/SUBFAC(I)
		FMWI=FMWI/SUBFAC(I)
		GOTO 221
	ENDIF
  222	CONTINUE
C
C	IF THIS IS ON LIST OF "EXPLICIT" MODEL SPECIES, THEN IT IS NOT
C	LUMPED.
C
	DO 226 I=1,NMODSP
	IF (NAMEIN.EQ.MODSP(I)) THEN
		IMODSP=I
		GOTO 230
	ENDIF
  226	CONTINUE
	IMODSP=0
C
C	DETERMINE WHICH MODEL SPECIES THIS IS (IF ANY)
C
  228	DO 229 IMOD=1,NAA
	IF (NAMEIN.EQ.AANAM(IMOD)) GOTO 240
  229	CONTINUE
C
C	NOT A MODEL SPECIES.  SEE IF IT IS ON LIST OF 'NON-LUMPED' 
C	SPECIES
C
  230	DO 234 I=1,NNLS
	IF (NAMEIN.EQ.NLSNAM(I)) THEN
		NLSLVL(I)=NLSLVL(I)+F
		NLSNC(I)=NLSNC(I)+F*FNCI
		NLSMAS(I)=NLSMAS(I)+F*FMWI
		GOTO 220
	ENDIF
  234	CONTINUE
C
C	NOT FOUND ON LIST OF NON-LUMPED SPECIES.  ADD IT.
C
	IF (NNLS.EQ.MAXNLS) STOP 'TOO MANY NON-LUMPED-PARM SPECIES'
	NNLS=NNLS+1
	NLSNAM(NNLS)=NAMEIN
	NLSLVL(NNLS)=F
	NLSNC(NNLS)=F*FNCI
	NLSMAS(NNLS)=F*FMWI
	NLSMOD(NNLS)=IMODSP
	IF (IMODSP.NE.0) GOTO 220
C
C	GIVE ERROR MESSAGE IF THIS NEW SPECIES IS NOT DECLARED AS
C	AN EXPLICIT MODEL SPECIES.  THEN ADD IT TO LIST WITH ZERO
C	MW.
C
	WRITE (6,*) 'Warning:  Unrecognized model species = "',
     &	 NAMEIN(1:LNBLNK(NAMEIN)),'" is being output.'
	IF (NMODSP.EQ.MMODSP) STOP
     &	 'Too many unrecognized model species.  Increase MMODSP'
	NMODSP=NMODSP+1
	MODSP(NMODSP)=NAMEIN
	MODNC(NMODSP)=0.0
	MODMW(NMODSP)=0.0
	GOTO 220
C
C
C	THIS IS MODEL SPECIES NO. IMOD.  FIND GROUP IT GOES WITH.
C	IGRP IS GROUP NO. WITH SAME TYPE ID, AND WITH HIGHEST MINIMUM
C	KOH WHICH IS STILL LESS THAN THIS SPECIES' KOH.  IGRP=0 IF
C	SOME GROUP HAS SAME ID, BUT NONE HAVE MINIMUM KOH LESS THAN THIS
C	ONE.  IGRP=-1 IF NO GROUP HAS SAME TYPE ID
C
  240	RKOH=AAKOH(IMOD)
	KODE=AAGRP(IMOD)
C
	IGRP=-1
	RKOHMX=-1.0
	DO 242 I=1,NGRP
	IF (KODE.NE.GRPTYP(I)) GOTO 242
	IF (IGRP.LT.0) IGRP=0
	IF (RKOH.GE.KOHMIN(I)) THEN
		IF (KOHMIN(I).GT.RKOHMX) THEN
			RKOHMX=KOHMIN(I)
			IGRP=I
		ENDIF
	ENDIF
  242	CONTINUE
C
	IF (IGRP.LT.0) THEN
C
C		ERROR IF NO GROUP WITH THIS TYPE ID
C
		IF (UNITA.NE.0) WRITE (UNITA,244) AANAM(IMOD)
		WRITE (ICRT,244) AANAM(IMOD)
  244		FORMAT ('! Error.  Undefined group for ',A4,'.')
C
	ELSEIF (IGRP.EQ.0) THEN
C
C		KOH LESS THAN MINIMUM FOR THIS GROUP.  THIS IS
C		'INERT'.  ITS PARMS ARE IGNORED.
C
		TOTGRP(1)=TOTGRP(1)+F
		NCGRP(1)=NCGRP(1)+(FNCI*F)
		MASGRP(1)=MASGRP(1)+(FMWI*F)
C
	ELSE
C
C		THIS IS LUMPED IN GROUP IGRP.  SUM UP AMOUNT,
C		KOH, PARMS.  F IS AMOUNT IN GROUP, WF IS AMOUNT
C		WEIGHED BY AMOUNT REACTED, AS DETERMINED BY
C		GRPIND AND/OR INTOH AND RKOH
C
		IF (GRPIND(IGRP).EQ.1) THEN
			WF=F*(1.0-EXP(-RKOH*INTOH))
		ELSEIF (GRPIND(IGRP).EQ.2) THEN
			WF=F*RKOH
		ELSEIF (GRPIND(IGRP).EQ.3) THEN
			IF (RKOH.GT.DRKGRP(IGRP)) THEN
				WF=F*(1.0-EXP(-DRKGRP(IGRP)*INTOH))
			ELSE
				WF=F*(1.0-EXP(-RKOH*INTOH))
			ENDIF
		ELSE
			WF=F
		ENDIF
		TOTGRP(IGRP)=TOTGRP(IGRP)+F
		NCGRP(IGRP)=NCGRP(IGRP)+(F*FNCI)
		NCRGRP(IGRP)=NCRGRP(IGRP)+(WF*FNCI)
		MASGRP(IGRP)=MASGRP(IGRP)+(F*FMWI)
		MSRGRP(IGRP)=MSRGRP(IGRP)+(WF*FMWI)
		RCTGRP(IGRP)=RCTGRP(IGRP)+WF
		RKGRP(IGRP)=RKGRP(IGRP)+WF*RKOH
		IF (GRPPNM(IGRP).NE.'  ') THEN
			DO 246 I=1,NAAPRM
  246			PRMGRP(I,IGRP)=PRMGRP(I,IGRP)
     &			 +WF*AAPARM(I,IMOD)
		ENDIF
C
	ENDIF
C
CD	WRITE (ICRT,*) RKOH,F,WF,GRPIND(IGRP)
C
  220	CONTINUE
C
C	END OF COMPOSITON INPUT PROCESSING.  DO LUMPING AND OUTPUT
C	RESULTS 
C
C
C	TOTLOS = TOTAL OF 'LOST CARBON'.  AFTER THIS POINT, NLSNC
C	AND NCGRP CHANGED TO MEAN TOTAL 'LOST' CARBON IN THE GROUPS
C	TLSMAS IS MASS
C
	TOTLOS=0.0
	TLSMAS=0.0
C
C
C	OUTPUT 'NON-LUMPED' SPECIES, IF ANY.  FIND OUT HOW MANY CARBONS
C	'LOST' BY CHECKING NO. CARBONS IN 'EXPLICIT' SPECIES.  FIGURE OUT
C	'LOST' MASS, AND PUT RESULTS IN NLSMAS
C
	DO 284 I=2,NNLS
	IF (NLSLVL(I).EQ.0.0) GOTO 284
	IF (UNITA.NE.0) THEN
		IF (LPRE.EQ.0) THEN
			WRITE (UNITA,'(A8,1PE12.4)') NLSNAM(I),NLSLVL(I)
		ELSE
			WRITE (UNITA,'(A,A,1PE12.4)') LPRECH(1:LPRE),
     &			 NLSNAM(I)(1:8-LPRE),NLSLVL(I)
		ENDIF
	ENDIF
C
C	NLSNC NOW LOST C FOR THIS SPECIES.  (NLSMOD=0 IF UNDEFINED, AND
C	GIVEN ZERO C'S.)
C
	IF (NLSMOD(I).NE.0) THEN
		NLSNC(I)=NLSNC(I)-NLSLVL(I)*MODNC(NLSMOD(I))
		NLSMAS(I)=NLSMAS(I)-(NLSLVL(I)*MODMW(NLSMOD(I)))
	ENDIF
C
	TOTLOS=TOTLOS+NLSNC(I)
	TLSMAS=TLSMAS+NLSMAS(I)
  284	CONTINUE

C
C	CALCULATE AVERAGES FOR LUMPED GROUPS, AND OUTPUT TOTAL
C	AMOUNTS AND AVERAGE PARMS.
C
C	FOR GROUPS WITH KOH WEIGHED AVERAGING (GRPIND NE 0), THE
C	EFFECTIVE INITIAL CONCENTRATION IS CALCULATED FROM THE
C	TOTAL AMOUNT REACTED, THE GROUP RATE CONSTANT, AND THE
C	INTEGRATED OH.  THE GROUP RATE CONSTANT IS EITHER THE
C	DEFAULT RATE CONSTANT (DRKGRP) READ FROM THE LUMPING CONTROL
C	FILE, OR (IF DRKGRP=0) THE WEIGHED AVERAGE RATE CONSTANT
C	FOR THIS GROUP.
C
C	FOR GROUPS WITH "SIMPLE" AVERAGING (GRPIND=0), DRKGRP
C	IS IGNORED, AND THE GROUP RATE CONSTANT IS ALWAYS THE
C	WEIGHED AVERAGE.
C
C	FNCI IS NUMBER OF CARBONS/MOLE OF REACTING SPECIES IN GROUP
C
C	NROG IS TOTAL MOLES C OF "INERT" PLUS "K < KMIN" (IGRP=1) LUMPED
C	SPECIES.  TOTAL C OF NON-REACTING SPECIES FOR GROUPS USING RATE
C	CONSTANT WEIGHING WILL BE ADDED TO THIS.  NROGMS IS MASS.
C
	NROG=NCGRP(1)+NLSNC(1)
	NROGMS=MASGRP(1)+NLSMAS(1)
C
	DO 300 IGRP=2,NGRP
	WF=RCTGRP(IGRP)
	IF (WF.EQ.0.0) GOTO 300
	RKOH=RKGRP(IGRP)/WF
	RKGRP(IGRP)=RKOH
	FNCI=NCRGRP(IGRP)/WF
	IF (GRPIND(IGRP).EQ.2) THEN
		IF (DRKGRP(IGRP).NE.0.0) RKOH=DRKGRP(IGRP)
		F=WF/RKOH
		XC=NCGRP(IGRP)-(F*FNCI)
		X=TOTGRP(IGRP)-F
		MSRGRP(IGRP)=MSRGRP(IGRP)*F/WF
		IF (X.NE.0.0) THEN
			XCX=XC/X
			FMWI=(MASGRP(IGRP)-MSRGRP(IGRP))/X
		ELSE
			XCX=0.0
			FMWI=0.0
		ENDIF
		IF (UNITB.NE.0) WRITE (UNITB,307) GRPNAM(IGRP),
     &		 F/TOTGRP(IGRP),TOTGRP(IGRP),NCGRP(IGRP),
     &		 NCGRP(IGRP)/TOTGRP(IGRP),MASGRP(IGRP)/TOTGRP(IGRP),F,
     &		 F*FNCI,FNCI,MSRGRP(IGRP)/F,X,XC,XCX,FMWI
  307		FORMAT ('!'/'!    GROUP = ',A4,',  PARAMETER AVERAGES'
     &           ,' WEIGHED BY OH RADICAL RATE'/'!                   '
     &           ,'CONSTANTS (SUITABLE FOR SLOWER REACTING SPECIES).'
     &           /'!                   FRACTION REACTING (MOLAR) ='
     &           ,F7.4/'!'/'!                 MOLES       MOLES C    C'
     &           ,'/MOLE    M.W.'/'!      TOTAL  ',1P2E12.4,0PF9.3,F9.2
     &           /'!     REACTING',1P2E12.4,0PF9.3,F9.2
     &           /'!      LOST   ',1P2E12.4,0PF9.3,F9.2/'!')
		IF (DRKGRP(IGRP).NE.0.0) THEN
			IF (UNITB.NE.0) WRITE (UNITB,306) RKGRP(IGRP),
     &			 RKOH
			RKGRP(IGRP)=DRKGRP(IGRP)
  306			FORMAT ('!',19X,'AVG KOH =',1PE11.4,
     &                   ', BUT DEFAULT OF',E11.4,' USED'/'!')
		ENDIF
	ELSEIF (GRPIND(IGRP).EQ.1.OR.GRPIND(IGRP).EQ.3) THEN
		IF (DRKGRP(IGRP).NE.0.0) RKOH=DRKGRP(IGRP)
		F=WF/(1.0-EXP(-RKOH*INTOH))
		XC=NCGRP(IGRP)-(F*FNCI)
		X=TOTGRP(IGRP)-F
		MSRGRP(IGRP)=MSRGRP(IGRP)*F/WF
		IF (X.NE.0.0) THEN
			XCX=XC/X
			FMWI=FMWI/X
			FMWI=(MASGRP(IGRP)-MSRGRP(IGRP))/X
		ELSE
			XCX=0.0
			FMWI=0.0
		ENDIF
		IF (UNITB.NE.0) WRITE (UNITB,308) GRPNAM(IGRP),INTOH,
     &		 F/TOTGRP(IGRP),TOTGRP(IGRP),NCGRP(IGRP),
     &		 NCGRP(IGRP)/TOTGRP(IGRP),MASGRP(IGRP)/TOTGRP(IGRP),F,
     &		 F*FNCI,FNCI,MSRGRP(IGRP)/F,X,XC,XCX,FMWI
  308		FORMAT ('!'/'!    GROUP = ',A4,',  PARAMETER AVERAGES'
     &           ,' WEIGHED BY ESTIMATED AMOUNT REACTED,'/'!          '
     &           ,'         BASED ON INTEGRATED OH =',1PE10.2,' PPM-MI'
     &           ,'N.'/'!                   FRACTION REACTING (MOLAR) ='
     &           ,0PF7.4/'!'/'!                 MOLES       MOLES C    '
     &           ,'C/MOLE    M.W.'/'!      TOTAL  ',1P2E12.4,0PF9.3,F9.2
     &           /'!     REACTING',1P2E12.4,0PF9.3,F9.2
     &           /'!      LOST   ',1P2E12.4,0PF9.3,F9.2/'!')
		IF (DRKGRP(IGRP).NE.0.0) THEN
			IF (GRPIND(IGRP).EQ.3.AND. UNITB.NE.0) WRITE (UNITB,*)
     &	 	 '!                   VOCs WITH KOH > DEFAULT LUMPED',
     &		 ' BY MOLE.'
			IF (UNITB.NE.0) WRITE (UNITB,306) RKGRP(IGRP),
     &			 RKOH
			RKGRP(IGRP)=DRKGRP(IGRP)
		ENDIF
	ELSE
		F=WF
		XC=0.0
		IF (UNITB.NE.0) WRITE (UNITB,309) GRPNAM(IGRP),
     &		 TOTGRP(IGRP),NCGRP(IGRP),NCGRP(IGRP)/TOTGRP(IGRP),
     &		 MASGRP(IGRP)/TOTGRP(IGRP)
  309		FORMAT ('!'/'!    GROUP = ',A4,',  PARAMETER AVERAGES'
     &           ,' WEIGHED BY AMOUNT PRESENT'/'!                   ('
     &           ,'SUITABLE FOR FASTER REACTING SPECIES).'
     &           /'!'/'!                 MOLES       MOLES C    C'
     &           ,'/MOLE    M.W.'/'!      TOTAL  ',1P2E12.4,0PF9.3,F9.2
     &           /'!')
	ENDIF
C
	NROG=NROG+XC
	NROGMS=NROGMS+MASGRP(IGRP)-MSRGRP(IGRP)
C
	NCGRP(IGRP)=XC
	RCTGRP(IGRP)=F
	NCRGRP(IGRP)=FNCI
C
	IF (UNITA.NE.0) THEN
		IF (LPRE.EQ.0) THEN
			WRITE (UNITA,'(A4,4X,1PE12.4)') 
     &			 GRPNAM(IGRP),F
		ELSE
			WRITE (UNITA,'(A,A,T9,1PE12.4)') LPRECH(1:LPRE),
     &			 GRPNAM(IGRP),F
		ENDIF
	ENDIF
C
	IF (UNITB.NE.0) THEN
		IF (VARYT) THEN
			IF (DRKGRP(IGRP).NE.0.0) THEN
				WRITE (UNITB,3101) GRPPNM(IGRP),'OH',
     &				 RKOH,DAGRP(IGRP),DEAGRP(IGRP),
     &				 DBGRP(IGRP)
			ELSE
				CALL ARRFIT3 (TLOW,TEMPR,THIGH,
     &				 PRMGRP(1,IGRP)/WF,RKOH,
     &				 PRMGRP(2,IGRP)/WF,A,EA)
				WRITE (UNITB,3101) GRPPNM(IGRP),'OH',
     &				 RKOH,A,EA
			ENDIF
		ELSE
			WRITE (UNITB,3101) GRPPNM(IGRP),'OH',RKOH
		ENDIF
 3101		FORMAT ('K(',2A2,') ',1P2E12.4,0P2F8.3)
		WRITE (UNITB,'(A2,''NC'',4X,1PE12.4)') GRPPNM(IGRP),FNCI
		DO 310 I=1,NAAPRM
		IF (PRMGRP(I,IGRP).EQ.0.0) GOTO 310
		IF (AAPRKI(I)) THEN
			IF (AAPNAM(I)(1:1).EQ.'*') GOTO 310
			IF (VARYT) THEN
				CALL ARRFIT3 (TLOW,TEMPR,THIGH,
     &				 PRMGRP(I+1,IGRP)/WF,PRMGRP(I,IGRP)/WF,
     &				 PRMGRP(I+2,IGRP)/WF,A,EA)
				WRITE (UNITB,3101) GRPPNM(IGRP),
     &				 AAPNAM(I),PRMGRP(I,IGRP)/WF,A,EA
			ELSE
				WRITE (UNITB,3101) GRPPNM(IGRP),
     &				 AAPNAM(I),PRMGRP(I,IGRP)/WF
			ENDIF
		ELSEIF (OUTRXN) THEN
			IF (AAPROD(I)(1:1).NE.'  ') THEN
				WRITE (UNITB,
     &				 '(A2,1X,A8,1PE12.4)')
     &				 AAPNAM(I)(1:2),AAPROD(I),
     &				 PRMGRP(I,IGRP)/WF
			ENDIF
		ELSE
			WRITE (UNITB,'(A2,A6,1PE12.4)')
     &			 GRPPNM(IGRP),AAPNAM(I),PRMGRP(I,IGRP)/WF
		ENDIF
  310		CONTINUE
	ENDIF
  300	CONTINUE
C
C	OUTPUT "NON-REACTING" CARBONS.  SUM OF "INERT", K<KMIN LUMPED
C	GROUPS, AND UNREACTING PORTION OF LUMPED GROUPS WITH RATE
C	CONSTANT WEIGHING.  ALSO FIGURE ITS PER-CARBON MOLECULAR WEIGHT
C
	IF (UNITA.NE.0) THEN
		IF (NROG.NE.0.0) THEN
			WRITE (UNITA,'(''!'')')
			IF (INTFMT) THEN
				WRITE (UNITA,'(''! NROG'',2X,1PE12.4)')
     &				 NROG
			ELSE
				WRITE (UNITA,'(''NROG'',4X,1PE12.4)')
     &				 NROG
			ENDIF
			WRITE (UNITA,322) NLSNC(1),NCGRP(1)
  322			FORMAT ('!'/'! BREAKDOWN OF ''NON-REACTING'' C:'
     &                   /'!'/'! INERT',1PE15.4/'! K < MIN',E13.4)
			DO 324 I=2,NGRP
			IF (NCGRP(I).NE.0.0) WRITE (UNITA,
     &			 '(''! '',A4,4X,1PE12.4)') GRPNAM(I),NCGRP(I)
  324			CONTINUE
		ENDIF
	ENDIF
C
C	OUTPUT NUMBER OF CARBONS "LOST" DUE TO SURROGATE SPECIES
C	REPRESENTATION
C
	IF (UNITA.NE.0) THEN
	    IF (TOTLOS.NE.0.0) THEN
		WRITE (UNITA,'(''!'')')
		IF (INTFMT) THEN
			WRITE (UNITA,'(''! LOST'',2X,1PE12.4)') TOTLOS
		ELSE
			WRITE (UNITA,'(''LOST'',4X,1PE12.4)') TOTLOS
		ENDIF
		WRITE (UNITA,326)
  326		FORMAT ('!'/'! BREAKDOWN OF ''LOST'' C:'/'!')
		DO 328 I=2,NNLS
		IF (NLSNC(I).NE.0.0) WRITE (UNITA,'(''! '',A8,1PE12.4)')
     &		 NLSNAM(I),NLSNC(I)
  328		CONTINUE
	    ENDIF
	ENDIF
C
C	WRITE SUMMARY OF EMITTED ROG SPECIES, I.E., EXPLICIT MODEL
C	SPECIES INCLUDED IN THE "SPECIES" INPUT IN THE LUMPING
C	CONTROL FILE, AND THE LUMPED GROUPS.  THIS IS SAME ORDERING
C	AS SPECIES LISTING AT START OF SAROAD ASSIGNMENT FILE
C	PRODUCED BY "SARSUB", AND IS USED TO INFORM THE AIRSHED
C	MODELING PROGRAM OF THE ORDERING OF EMITTED ROG SPECIES
C	IN PROCESSED EMISSIONS DATA CREATED BASED ON THE SAROAD
C	ASSIGNMENT FILE.
C
	IF (UNITB.NE.0) WRITE (UNITB,
     &		 '(''!''/''! SUMMARY OF EMITTED ROG SPECIES:''/''!'')')
C
C	LIST EXPLICIT MODEL SPECIES.  IF MODEL SPECIES IS "NROG" OR
C	"LOST", USE MW AND NC VALUES CALCULATED FOR IT, RATHER THAN
C	THOSE INPUT FOR THEM IN THE "SPECIES" INPUT.
C
	IF (INTFMT) THEN
		IF (UNITB.NE.0) THEN
			DO 340 I=1,NMODSP
  340			WRITE (UNITB,342) MODSP(I)(1:4),MODNC(I),
     &			 MODMW(I)
  342			FORMAT ('!   ',A4,F7.2,F9.2)
		ENDIF
	ELSE
C
		DO 330 I=1,NMODSP
		IF (MODSP(I)(1:4).EQ.'NROG') THEN
			IF (NROG.NE.0.0) NROG=NROGMS/NROG
			IF (UNITB.NE.0) WRITE 
     &			 (UNITB,'(''RHC(NROG)  1.00'',F9.2)') NROG
			MODNC(I)=1.0
			MODMW(I)=NROG
			NROG=0.0
		ELSEIF (MODSP(I)(1:4).EQ.'LOST') THEN
			IF (TOTLOS.NE.0.0) TOTLOS=TLSMAS/TOTLOS
			IF (UNITB.NE.0) WRITE 
     &			 (UNITB,'(''RHC(LOST)  1.00'',F9.2)') TOTLOS
			MODNC(I)=1.0
			MODMW(I)=TOTLOS
			TOTLOS=0.0
		ELSE
			IF (UNITB.NE.0) WRITE (UNITB,332) MODSP(I)(1:4),
     &			 MODNC(I),MODMW(I)
  332			FORMAT ('RHC(',A4,')',F6.2,F9.2)
		ENDIF
  330		CONTINUE
	ENDIF
C
C	LIST LUMPED GROUPS.  CHANGE MSRGRP TO GROUP MOLECULAR WEIGHT.
C
	IF (INTFMT) THEN
		DO 344 I=2,NGRP
		IF (RCTGRP(I).NE.0.0) THEN
			MSRGRP(I)=MSRGRP(I)/RCTGRP(I)
			IF (UNITB.NE.0) WRITE (UNITB,342) GRPNAM(I),
     &			 NCRGRP(I),MSRGRP(I)
		ELSEIF (UNITB.NE.0) THEN
			WRITE (UNITB,342) GRPNAM(I)
		ENDIF
  344		CONTINUE
C
	ELSE
C
		DO 334 I=2,NGRP
		IF (RCTGRP(I).NE.0) THEN
			MSRGRP(I)=MSRGRP(I)/RCTGRP(I)
			IF (UNITB.NE.0) WRITE (UNITB,332) GRPNAM(I),
     &			 NCRGRP(I),MSRGRP(I)
		ELSEIF (UNITB.NE.0) THEN
			WRITE (UNITB,332) GRPNAM(I)
		ENDIF
  334		CONTINUE
	ENDIF
C
	IF (UNITB.NE.0) THEN
		IF (NROG.NE.0.0) WRITE (UNITB,
     &		 '(''!   NROG   1.00'',F9.2)') NROGMS/NROG
		IF (TOTLOS.NE.0.0) WRITE (UNITB,
     &	 	 '(''!   LOST   1.00'',F9.2)') TLSMAS/TOTLOS
	ENDIF
C
C
C	DONE PROCESSING LUMPING FOR THIS EMISSION FILE.
C
	RETURN
	END
