C	RDGENP.FTN  (VERSION 2 -- CAN INCLUDE ALKENES)
C
C	READS MODEL SPECIES AND KINETIC AND MECHANISTIC PARAMETERS IN THE
C	GENERAL, OR "AROMATIC" FORMAT.
C
C	ALSO INCLUDES RDGN2P.FOR, FOR READING ".GEN" FILES
C	(GEN2=.TRUE.)
C
	SUBROUTINE RDGENP(FUNIT)
C
C	ARGUMENT:  FILE UNIT FOR INPUT
C
	INTEGER FUNIT
	LOGICAL GEN2
C
	INCLUDE 'LUMPSPEC.FOR'
C
        PARAMETER (MAXT=10)
	LOGICAL ERRIND,RKPPM,EOF
	REAL TEMPS(MAXT)
C
	CHARACTER*6 AROPBF(MXAAPM),AROPNM
	CHARACTER*8 AROPB2(MXAAPM)
	CHARACTER INLIN*120,NAMEIN*8
	LOGICAL VARYT
C
C	RDGENP:  ENTRY FOR ".LPM" FINPUT FORMAT
C
	GEN2=.FALSE.
	GOTO 2
C
C
C	RDGN2P:  ENTRY POINT FOR ".GEN" INPUT FORMAT.
C
	ENTRY RDGN2P (FUNIT)
	GEN2=.TRUE.
    2	CONTINUE

	EOF=.FALSE.
C
C	OPTION TO INPUT K'S IN PPM-MIN UNITS FOR 300K.
C
	RKPPM=.FALSE.
C
C	INITIALIZE TEMPERATURE-DEPENDENT VARIABLES
C
        RTEMP=0.0019872*TEMPR
        CTEMP=4.40373E+17/TEMPR
	BTEMP=TEMPR/300.0
	VARYT=(THIGH.NE.0.0.AND.TLOW.NE.0.0)
C
C	FIRST 2 PARMS RESERVED FOR KOH AT TLOW AND THIGH, RESPECTIVELY.
C
	IF (VARYT) THEN
		RTEMP1=0.0019872*TLOW
		RTEMP2=0.0019872*THIGH
	        CTEMP1=4.40373E+17/TLOW
	        CTEMP2=4.40373E+17/THIGH
		BTEMP1=TLOW/300.0
		BTEMP2=THIGH/300.0
		IF (NAAPRM.EQ.0) THEN
			NAAPRM=2
			AAPNAM(1)='*1'
			AAPNAM(2)='*2'
			AAPRKI(1)=.TRUE.
			AAPRKI(2)=.TRUE.
			AAPROD(1)=' '
			AAPROD(2)=' '
		ENDIF
	ENDIF		
C
C	READ TEMPERATURES FOR WHICH PARMS ARE GIVEN.  300K USED
C	IF NONE GIVEN.
C
	KODE=3001
	INLIN='<EOF>'
	READ (FUNIT,'(A)',END=4380) INLIN
	CALL PARSE0 (MAXT+3,INLIN,80)
	CALL PARSI2 (2,NTEMPS,0,ERRIND,INLIN)
	IF (ERRIND.OR.NTEMPS.LE.1) NTEMPS=1
	CALL PARSR2 (3,TEMPS(1),300.0,ERRIND,INLIN)
	IF (ERRIND) GOTO 4380
	IF (NTEMPS.GT.1) THEN
		DO 110 I=2,MIN0(NTEMPS,MAXT)
		CALL PARSER (2+I,TEMPS(I),ERRIND,INLIN)
		IF (ERRIND.OR.TEMPS(I).EQ.0.0) THEN
			NTEMPS=I-1
			GOTO 111
		ENDIF
		IF (TEMPS(I).LE.TEMPS(I-1)) THEN
			WRITE (ICRT,*) 'Bad T order in general',
     &			 ' parm data set.'
			GOTO 4380
		ENDIF
  110		CONTINUE
C
C		SEE IF PRESSURE-DEPENDENT PARMS.  IF SO UPDATE ATM.
C		ABORT OF DISAGREES WITH VALUE READ IN BEFORE.
C
		IF (NTEMPS.GT.MAXT) GOTO 111
		CALL PARSER (NTEMPS+3,F,ERRIND,INLIN)
		IF (.NOT.(ERRIND.OR.F.LE.0.0)) THEN
			IF (ATM.EQ.0) THEN
				ATM=F
			ELSEIF (ATM.NE.F) THEN
				WRITE (ICRT,*) 'Pressure in general',
     &				 ' parm data set disagrees with pr',
     &				 'evious value.'
				GOTO 4380
			ENDIF
		ENDIF
C
  111		CONTINUE
		IF (NTEMPS.GT.MAXT) THEN
			WRITE (ICRT,
     &			 '('' Warning.  Max T = '',F8.2,'' used.'')')
     &			 TEMPS(MAXT)
			NTEMPS=MAXT
		ENDIF
C
C       	GET THE INFO NEEDED TO EXTRAPOLATE THE TEMPERATURE-DEPENDENT
C       	PARMS TO TEMP=TEMPR.  DON'T EXTRAPOLATE IF ONLY ONE T.
C
		I2=0
		IF (TEMPR.LT.TEMPS(1)) THEN
			GOTO 3186
		ELSEIF (TEMPR.EQ.TEMPS(1)) THEN
			I1=1
			GOTO 121
		ENDIF
		DO 120 I=2,NTEMPS
		IF (TEMPR.EQ.TEMPS(I)) THEN
			I1=I
			GOTO 121
		ELSEIF (TEMPR.LT.TEMPS(I)) THEN
			I2=I
			I1=I-1
	                FT2=TEMPS(I2)-TEMPS(I1)
	                FT1=(TEMPS(I2)-TEMPR)/FT2
	                FT2=(TEMPR-TEMPS(I1))/FT2
			GOTO 121
		ENDIF
  120		CONTINUE
		GOTO 3186
  121		CONTINUE
        ENDIF
C
C	READ LIST OF PARMS TO BE USED.  (FIRST RESERVED FOR O3, NO3 (N3),
C	AND O(3P) (OA) HNO3 (HN) RATE CONSTANTS.
C
	KODE=3002
	IF (VARYT) THEN
		NKP=18
		AROPBF(1)='O3'
		AROPBF(2)='*3'
		AROPBF(3)='*4'
		AROPBF(4)='N3'
		AROPBF(5)='*5'
		AROPBF(6)='*6'
		AROPBF(7)='OA'
		AROPBF(8)='*7'
		AROPBF(9)='*8'
		AROPBF(10)='OP'
		AROPBF(11)='*9'
		AROPBF(12)='*A'
		AROPBF(13)='CL'
		AROPBF(14)='*B'
		AROPBF(15)='*C'
		AROPBF(16)='HN'
		AROPBF(17)='*D'
		AROPBF(18)='*E'
	ELSE
		NKP=6
		AROPBF(1)='O3'
		AROPBF(2)='N3'
		AROPBF(3)='OA'
		AROPBF(4)='OP'
		AROPBF(5)='CL'
		AROPBF(6)='HN'
	ENDIF
	IF (GEN2) THEN
		NPARO=NKP
		DO 1221 I=1,NKP
 1221		AROPB2(I)=' '
  122		READ (FUNIT,'(A)',ERR=4380) INLIN
		IF (INLIN(1:1).NE.' ') THEN
			IF (INLIN(1:6).EQ.'PARMS ') THEN
				CALL PARSE0 (2,INLIN,80)
				CALL PARSEI (2,N,ERRIND,INLIN)
				IF (ERRIND.OR.N.LE.0) GOTO 3006
				NPARO=NKP+N
				IF (NPARO.GT.MXAAPM) THEN
					WRITE (ICRT,*)
     &					 'INCREASE MXAAPM IN LUMPSPEC'
					GOTO 9999
				ELSEIF (N.GT.0) THEN
					OUTRXN=.FALSE.
					READ (FUNIT,'(8(A6,4X))') 
     &					 (AROPBF(I),I=NKP+1,NKP+N)
					GOTO 3004
				ENDIF
			ENDIF
 3006			IF (OUTRXN) THEN
				CALL PARSE0 (2,INLIN,80)
				CALL PARSE1 (1,NAMEIN,I,6,INLIN)
			ELSE
				NAMEIN=INLIN
			ENDIF
			IF (NPARO.EQ.MXAAPM) THEN
				WRITE (ICRT,*) 'INCREASE MXAAPM IN LUMPSPEC'
				GOTO 9999
			ENDIF
			NPARO=NPARO+1
			AROPBF(NPARO)=NAMEIN(1:6)
			IF (OUTRXN) THEN
				CALL PARSE1 (2,NAMEIN,I,8,INLIN)
				AROPB2(NPARO)=NAMEIN
			ELSE
				AROPB2(NPARO)=' '
			ENDIF
			GOTO 122
		ENDIF
	ELSE
	        READ (FUNIT,3002,ERR=4380) NPARO,(AROPBF(I+NKP),I=1,
     &		MIN0(MXAAPM-NKP,NPARO))
 3002   	FORMAT (I2,1X,100(A2,1X))
		DO 3003 I=NPARO+1,NPARO+NKP
 3003		AROPB2(I)=' '
		NPARO=NPARO+NKP
		IF (NPARO.GT.MXAAPM) THEN
			WRITE (ICRT,*) 'INCREASE MXAAPM IN LUMPSPEC'
			GOTO 9999
		ENDIF
	ENDIF
 3004	CONTINUE
C
C       COMBINE NEW PARMS WITH OLD ONES, IF ANY
C
	NOLD=NAAPRM
	DO 4100 I=1,NPARO
	AROPNM=AROPBF(I)
	IF (NAAPRM.GT.0) THEN
		DO 4102 J=1,NOLD
		IF (AROPNM.EQ.AAPNAM(J)) THEN
			IF (I.GT.NKP.OR.AAPRKI(J)) GOTO 4100
			WRITE (ICRT,*) 'Error.  "',AROPNM,
     &			 '" used for non-kinetic parameter name'
			CALL EXIT (1)
			IF (OUTRXN.AND.AROPNM(3:3).NE.' ') THEN
				IF (AAPROD(J).NE.AROPB2(I)) THEN
					WRITE (ICRT,*) 'Inconsistent ',
     &					 'product species (',
     &					 AROPB2(I),') for parameter = '
     &					 ,AROPNM
					CALL EXIT (1)
				ENDIF
			ENDIF
		ENDIF
 4102		CONTINUE
	ENDIF
C
C	NEW PARM.  ADD TO LIST, AND SET ITS VALUE TO ZERO FOR
C	MODEL SPECIES ALREADY INPUT
C
	NAAPRM=NAAPRM+1
	AAPROD(NAAPRM)=' '
	IF (NAAPRM.GT.MXAAPM) THEN
		WRITE (ICRT,*) 'INCREASE MXAAPM IN LUMPSPEC'
		CALL EXIT (2)
	ENDIF
	AAPNAM(NAAPRM)=AROPBF(I)
	IF (OUTRXN) AAPROD(NAAPRM)=AROPB2(I)
	IF (NAA.GT.0) THEN
		DO 4104 J=1,NAA
 4104		AAPARM(NAAPRM,J)=0.0
	ENDIF
	AAPRKI(NAAPRM)=I.LE.NKP
 4100	CONTINUE
C
C
C       READ NAME, A, EA
C
 4300   READ (FUNIT,'(A)',END=4398) INLIN
	IF (INLIN(1:1).EQ.'!') GOTO 4300
        IF (INLIN(1:1).EQ.' ') GOTO 4300
	IF (INLIN(1:1).EQ.'*') THEN
		IF (INLIN(2:4).EQ.'PPM') THEN
			RKPPM=.TRUE.
		ELSEIF (INLIN(2:6).EQ.'NOPPM') THEN
			RKPPM=.FALSE.
		ELSE
			WRITE (ICRT,*) 'Unrecognized option = ',
     &			 INLIN(1:LNBLNK(INLIN))
			GOTO 4380
		ENDIF
		GOTO 4300
	ENDIF
        IF (NAA.EQ.MAXAA) THEN
		WRITE (ICRT,*) 'INCREASE MAXAA IN LUMPSPEC'
		CALL EXIT (2)
	ENDIF
        NAA=NAA+1
	KODE=4303
	IF (GEN2) THEN
		AANAM(NAA)=INLIN(1:8)
		AROA=0.0
		AROEA=0.0
		AROB=0.0
	ELSE
		READ (INLIN,4303,ERR=4380) AANAM(NAA),AROA,AROEA,AROB
 4303   	FORMAT (A8,T31,3F10.0)
	ENDIF
	IF (RKPPM) THEN
		AROA=AROA/CTEMP
		AROB=AROB+1.0
	ENDIF
C
C       INITIALIZE PARMS
C
        DO 4310 I=1,NAAPRM
 4310   AAPARM(I,NAA)=0.
C
C
C       READ PARM LABELS, VALUES, AND PUT IN PLACE.  BLANK = END

 4340   READ (FUNIT,'(A)',END=4390) INLIN
	IF (INLIN(1:1).EQ.'!') GOTO 4340
        IF (INLIN(1:1).EQ.' ') GOTO 4392
	IF (INLIN(1:5).EQ.'K(HV)'.OR.INLIN(1:4).EQ.'kHV ') GOTO 4340
C
C	IF "*DMSNAME" CODE USED, READ PARMS FROM PREVIOUSLY READ DMS,
C	USING FACTOR INPUT.
C	NOTE:  REFERENCING PREVIOUS DMS WITH A FACTOR ADDS PARM TO PREVIOUSLY
C	INPUT VALUE, WHILE REFERENCING PARM BY NAME OVER-WRITES
C	IT.  REFERENCING PREVIOUS DMS W/O A FACTOR OVER-WRITES ALL PARMS.
C
	IF (INLIN(1:1).EQ.'*') THEN
		KODE=4350
		IF (INLIN(2:2).EQ.' ') GOTO 4380
		KODE=4352
		INLIN(1:1)=' '
		CALL PARSE0 (2,INLIN,80)
		CALL PARSE1 (1,NAMEIN,I,8,INLIN)
		CALL PARSR2 (2,F,-999.,ERRIND,INLIN)
		IF (ERRIND) GOTO 4380
C
C		IF NO FACTOR, VOC GIVEN ALL PARAMETERS OF REFERENCED
C		VOC.  IF A FACTOR IS GIVEN, KINETIC PARMS ARE IGNORED.
C
		IF (F.EQ.-999.) THEN
			IF (NAA.GT.1) THEN
				DO 4355 I=1,NAA-1
				IF (NAMEIN.EQ.AANAM(I)) THEN
					DO 4356 J=1,NAAPRM
 4356					AAPARM(J,NAA)=AAPARM(J,I)
					GOTO 4340
				ENDIF
 4355				CONTINUE
			ENDIF
			GOTO 4359
		ENDIF
C
		KODE=4353
		IF (F.EQ.0.0) GOTO 4380
		IF (NAA.GT.1) THEN
			DO 4350 I=1,NAA-1
			IF (NAMEIN.EQ.AANAM(I)) THEN
				DO 4352 J=1,NAAPRM
				IF (.NOT.AAPRKI(J)) AAPARM(J,NAA)=
     &				 AAPARM(J,NAA)+F*AAPARM(J,I)
 4352				CONTINUE
				GOTO 4340
			ENDIF
 4350			CONTINUE
		ENDIF
 4359		WRITE (ICRT,*) 'UNKNOWN DMS = ',NAMEIN,' REFERENCED.'
		KODE=4354
		GOTO 4380
C
	ELSEIF (INLIN(1:2).EQ.'K(') THEN
C
C		KINETIC PARAMETER INPUT
C
		CALL PARSE0 (4,INLIN,80)
C	      - IF THERE IS AN ERROR READING EA OR B, IT IS ASSUMED
C	      - THEY ARE ZERO, AND THE ERROR IS DUE TO A COMMENT.
		CALL PARSER (2,A,ERRIND,INLIN)
		IF (ERRIND) GOTO 5282
		CALL PARSR2 (3,EA,0.0,ERRIND,INLIN)
		IF (ERRIND) THEN
			EA=0.0
			B=0.0
		ELSE
			CALL PARSR2 (4,B,0.0,ERRIND,INLIN)
			IF (ERRIND) B=0.0
		ENDIF
		IF (RKPPM) THEN
			A=A*6.8125E-16
			B=B+1.0
		ENDIF
		IF (INLIN(3:4).EQ.'OH') THEN
			AROA=A
			AROB=B
			AROEA=EA
			GOTO 4340
		ELSE
			DO 5200 I=1,NAAPRM
			IF (INLIN(3:4).EQ.AAPNAM(I)) THEN
			    IF (TOPPM) THEN
				AAPARM(I,NAA)=CTEMP*A*(BTEMP**B)*
     &				 EXP(-EA/RTEMP)
				IF (VARYT) THEN
					AAPARM(I+1,NAA)=CTEMP1*
     &					 A*(BTEMP1**B)*EXP(-EA/RTEMP1)
					AAPARM(I+2,NAA)=CTEMP2*
     &					 A*(BTEMP2**B)*EXP(-EA/RTEMP2)
				ENDIF
			     ELSE
				AAPARM(I,NAA)=A*(BTEMP**B)*EXP(-EA/RTEMP)
				IF (VARYT) THEN
					AAPARM(I+1,NAA)=
     &					 A*(BTEMP1**B)*EXP(-EA/RTEMP1)
					AAPARM(I+2,NAA)=
     &					 A*(BTEMP2**B)*EXP(-EA/RTEMP2)
				ENDIF
			  ENDIF
				GOTO 4340
			ENDIF
 5200			CONTINUE
			WRITE (ICRT,*) 'Unrecognized kinetic parm: ',
     &			 INLIN(1:8)
			CALL EXIT (2)
		ENDIF
C
	ELSE
C
		KODE=4355
		CALL PARSE0 (NTEMPS+1,INLIN,80)
		CALL PARSE1 (1,NAMEIN,I,8,INLIN)
	        DO 4342 I=1,NAAPRM
	        IF (NAMEIN(1:6).EQ.AAPNAM(I)) THEN
			IF (NTEMPS.EQ.1) THEN
				CALL PARSER (2,F,ERRIND,INLIN)
				IF (ERRIND) GOTO 4380
			ELSE
				CALL PARSER (I1+1,F,ERRIND,INLIN)
				IF (ERRIND) THEN
					IF (I1.EQ.1) GOTO 4380
					CALL PARSER (2,F,ERRIND,INLIN)
					IF (ERRIND) GOTO 4380
				ELSEIF (I2.NE.0) THEN
					CALL PARSER (I2+1,F2,ERRIND,
     &					 INLIN)
					IF (ERRIND) THEN
						IF (I1.NE.1) GOTO 4380
					ELSE
						F=(F*FT1)+(F2*FT2)
					ENDIF
				ENDIF
			ENDIF
	                AAPARM(I,NAA)=AAPARM(I,NAA)+F
	                GOTO 4340
	        ENDIF
 4342   	CONTINUE
		GOTO 4382
	ENDIF
C
C	ERRORS
C
 4380	WRITE (ICRT,*) 'Error reading data from general format ',
     &	 'lumping file.'
	IF (NAA.EQ.0) THEN
		WRITE (ICRT,*) 'No species successfully processed'
	ELSE
		WRITE (ICRT,*) 'Last species processed = ',AANAM(NAA)
	ENDIF
	WRITE (ICRT,*) 'RDGENP Code = ',KODE
	WRITE (ICRT,*) 'INLIN = ''',INLIN(1:LNBLNK(INLIN)),''''
	CALL EXIT (2)
C
 3186	WRITE (ICRT,3185) TEMPR
 3185	FORMAT (' Aborted because temperature out of range in gen',
     &   'eral parm data set.  TEMPR=',F6.2)
	CALL EXIT (2)
C
 4382	WRITE (ICRT,4384) NAMEIN(1:6),AANAM(NAA)
 4384   FORMAT (' Unrecognized parm = ''',A,''' for ',A8,
     &   ' in general parameter data set.')
	WRITE (ICRT,*) NAAPRM,' parms used:'
	WRITE (ICRT,'(10(1X,A6))') (AAPNAM(I),I=1,NAAPRM)
C
 5282   WRITE (ICRT,*) 'Error reading kinetic parameter input',
     &	 '.  Input line is:'
	WRITE (ICRT,*) INLIN
        CALL EXIT (2)
C
C
C	END OF INPUT
C
C
 4390	CLOSE (UNIT=FUNIT)
 4391	EOF=.TRUE.
C
C	CALCULATE K'S
C
 4392	IF (TOPPM) THEN
		AAKOH(NAA)=CTEMP*AROA*(BTEMP**AROB)*EXP(-AROEA/RTEMP)
		IF (VARYT) THEN
			AAPARM(1,NAA)=CTEMP1*AROA*(BTEMP1**AROB)
     &			 *EXP(-AROEA/RTEMP1)
			AAPARM(2,NAA)=CTEMP2*AROA*(BTEMP2**AROB)
     &			 *EXP(-AROEA/RTEMP2)
		ENDIF
	ELSE
		AAKOH(NAA)=AROA*(BTEMP**AROB)*EXP(-AROEA/RTEMP)
		IF (VARYT) THEN
			AAPARM(1,NAA)=AROA*(BTEMP1**AROB)
     &			 *EXP(-AROEA/RTEMP1)
			AAPARM(2,NAA)=AROA*(BTEMP2**AROB)
     &			 *EXP(-AROEA/RTEMP2)
		ENDIF
	ENDIF
	IF (EOF) RETURN
	GOTO 4300
C
 4398	CLOSE (UNIT=FUNIT)
	RETURN
C
C	ABORT
C
 9999	CALL EXIT (2)
        END
